from typing import Optional
from pydantic import BaseModel, Field


class PagedAndSortedResultRequestDto(BaseModel):
    sorting: Optional[str] = None
    skipCount: Optional[int] = None
    maxResultCount: Optional[int] = None



"""constant"""


import sys

AccessToken = None

IsDebug = bool(sys.gettrace())
if IsDebug:
    AuthTokenUrl = "https://localhost:44319/connect/token"
    Auth_Info = {
        "client_id": "Sparks_App",
        "scope": "Sparks",
        "grant_type": "password",
        "username": "admin",
        "password": "1q2w3E*",
    }

    ApiUrl = "https://localhost:44319/api"

else:
    AuthTokenUrl = "https://wwww.sparkstrader.com/connect/token"
    Auth_Info = {
        "client_id": "Sparks_App",
        "scope": "Sparks",
        "grant_type": "password",
        "username": "admin",
        "password": "1q2w3E*",
    }

    ApiUrl = "https://wwww.sparkstrader.com/api"



from dataclasses import dataclass
from typing import Optional
from pydantic import BaseModel, Field
from sparkstrader.Enums.Enums import *


# @dataclass
class CreateUpdateExchangeDto(BaseModel):
    code: str = Field(..., min_length=1, max_length=32)
    name: str = Field(..., max_length=128)
    district: EDistrict
    currency: ECurrency
    iconPath: Optional[str] = Field(None, max_length=256)



from json.decoder import JSONDecodeError
import sys
import requests
from requests import get as requests_get
from requests import ConnectionError as requests_ConnectionError
from requests import Timeout as requests_Timeout
from requests.exceptions import HTTPError as requests_HTTPError
from rich.console import Console
import sys


class BaseApi:
    def __init__(self, apiUrl: str, accessToken: str) -> None:
        self.console = Console()
        self.accessToken = accessToken
        self.apiUrl = apiUrl

    def _rest_get_method(self, endpoint: str, uri: str = None, params: dict = None):
        """Generic REST GET"""

        if not endpoint.strip():
            raise ValueError("endpoint is empty!")

        try:
            head = {"Authorization": f"Bearer {self.accessToken}"}
            url = f"{self.apiUrl}/{endpoint}"
            # if uri
            if uri is not None:
                url = f"{url}/{uri}"

            resp = requests_get(url, headers=head, params=params, verify=False)
            path = resp.request.path_url

            if resp.status_code != 200:
                try:
                    if "message" in resp.json():
                        resp_message = resp.json()["message"]
                    elif "error" in resp.json():
                        self.console.log(resp.json())
                        sys.exit(1)
                    else:
                        resp_message = ""

                    message = f"({resp.status_code}) {self.apiUrl} - {resp_message}"
                    self.console.log(message)

                except JSONDecodeError as err:
                    self.console.log(err)

            try:
                resp.raise_for_status()

                # return resp.text
                return resp.json()

            except ValueError as err:
                self.console.log(err)

        except requests_ConnectionError as err:
            self.console.log(err)
        except requests_HTTPError as err:
            self.console.log(err)
        except requests_Timeout as err:
            self.console.log(err)
        return {}

    def _rest_post_method(self, endpoint: str, uri: str = None, json: dict = None):
        """Generic REST post"""

        if not endpoint.strip():
            raise ValueError("endpoint is empty!")

        try:
            head = {"Authorization": f"Bearer {self.accessToken}"}
            url = f"{self.apiUrl}/{endpoint}"
            # if uri
            if uri is not None:
                url = f"{url}/{uri}"

            resp = requests.post(url, headers=head, json=json, verify=False)

            if resp.status_code != 200:
                try:
                    if "message" in resp.json():
                        resp_message = resp.json()["message"]
                    elif "error" in resp.json():
                        self.console.log(resp.json())
                        sys.exit(1)
                    else:
                        resp_message = ""

                    message = f"({resp.status_code}) {self.apiUrl} - {resp_message}"
                    self.console.log(message)

                except JSONDecodeError as err:
                    self.console.log(err)

            try:
                resp.raise_for_status()

                # return resp.text
                return resp.json()

            except ValueError as err:
                self.console.log(err)

        except requests_ConnectionError as err:
            self.console.log(err)
        except requests_HTTPError as err:
            self.console.log(err)
        except requests_Timeout as err:
            self.console.log(err)
        return {}

    def _rest_put_method(self, endpoint: str, uri: str = None, json: dict = None):
        """Generic REST put"""

        if not endpoint.strip():
            raise ValueError("endpoint is empty!")

        try:
            head = {"Authorization": f"Bearer {self.accessToken}"}
            url = f"{self.apiUrl}/{endpoint}"
            # if uri
            if uri is not None:
                url = f"{url}/{uri}"

            resp = requests.put(url, headers=head, json=json, verify=False)

            if resp.status_code != 200:
                try:
                    if "message" in resp.json():
                        resp_message = resp.json()["message"]
                    elif "error" in resp.json():
                        self.console.log(resp.json())
                        sys.exit(1)
                    else:
                        resp_message = ""

                    message = f"({resp.status_code}) {self.apiUrl} - {resp_message}"
                    self.console.log(message)

                except JSONDecodeError as err:
                    self.console.log(err)

            try:
                resp.raise_for_status()

                # return resp.text
                return resp.json()

            except ValueError as err:
                self.console.log(err)

        except requests_ConnectionError as err:
            self.console.log(err)
        except requests_HTTPError as err:
            self.console.log(err)
        except requests_Timeout as err:
            self.console.log(err)
        return {}

    def _rest_delete_method(self, endpoint: str, uri: str = None):
        """Generic REST delete"""

        if not endpoint.strip():
            raise ValueError("endpoint is empty!")

        try:
            head = {"Authorization": f"Bearer {self.accessToken}"}
            url = f"{self.apiUrl}/{endpoint}"
            # if uri
            if uri is not None:
                url = f"{url}/{uri}"

            resp = requests.delete(url, headers=head, verify=False)

            # 204 代表成功
            if resp.status_code != 204:
                try:
                    if "message" in resp.json():
                        resp_message = resp.json()["message"]
                    elif "error" in resp.json():
                        self.console.log(resp.json())
                        sys.exit(1)
                    else:
                        resp_message = ""

                    message = f"({resp.status_code}) {self.apiUrl} - {resp_message}"
                    self.console.log(message)

                except JSONDecodeError as err:
                    self.console.log(err)

            try:
                resp.raise_for_status()

                # return resp.text
                return True

            except ValueError as err:
                self.console.log(err)

        except requests_ConnectionError as err:
            self.console.log(err)
        except requests_HTTPError as err:
            self.console.log(err)
        except requests_Timeout as err:
            self.console.log(err)
        return False



from enum import IntEnum


class EDistrict(IntEnum):
    Unknown = 0
    CN = 1
    HK = 2
    TW = 4
    US = 8


class ECurrency(IntEnum):
    Unknown = 0
    AUD = 1
    CAD = 2
    CHF = 4
    CNY = 8
    EUR = 16
    GBP = 32
    JPY = 64
    NZD = 128
    USD = 256
    HKD = 512
    PLY = 1024
    CalendarCurrencyAll = 1023



from .BaseApi import BaseApi
import pandas as pd
from uuid import UUID
import sys

from typing import TypeVar, Generic
from pydantic import BaseModel

TEntityDto = TypeVar("TEntityDto", bound=BaseModel)
TCreateInput = TypeVar("TCreateInput", bound=BaseModel)
TUpdateInput = TypeVar("TUpdateInput", bound=BaseModel)
TGetListInput = TypeVar("TGetListInput", bound=BaseModel)
TKey = TypeVar("TKey", bound=UUID)


class CrudApi(
    Generic[
        TKey,
        TEntityDto,
        TGetListInput,
        TCreateInput,
        TUpdateInput,
    ],
    BaseApi,
):
    """
    Login indicate if logined.
    """

    _TEntityDto: type  # 因为typevar在运行时其实并不存在，需要在init的时候提供一个type变量，以用于instantiate

    def __init__(
        self, apiUrl: str, accessToken: str, endpoint: str, tEntityDto: type
    ) -> None:
        BaseApi.__init__(self, apiUrl, accessToken)
        self.endpoint = endpoint
        self._TEntityDto = tEntityDto

    def Get(self, id: TKey) -> TEntityDto:
        """
        Get the entity with the specified ID.

        Args:
            id: The ID of the entity.

        Returns:
            TEntityDto: The entity object.

        Raises:
            None.

        Examples:
            >>> api = CrudApi()
            >>> entity = api.Get(1)
        """
        r = self._rest_get_method(endpoint=self.endpoint, uri=id)
        # print(type(r))
        # print(r)
        return self._TEntityDto(**r)

    def GetList(self, dto: TGetListInput = None) -> pd.DataFrame:
        params = dto.model_dump() if dto is not None else None
        print(params)
        r = self._rest_get_method(endpoint=self.endpoint, params=params)
        l = r["items"]
        df = pd.DataFrame(l)
        # from_list(ExchangeDto.from_dict, r.get("ExchangeDto"))
        print(type(r))
        print(r)
        return df

    def Create(self, dto: TCreateInput) -> TEntityDto:
        json = dto.model_dump() if dto is not None else None
        # print(req.model_dump())
        r = self._rest_post_method(endpoint=self.endpoint, json=json)
        print(type(r))
        print(r)

        r = self._TEntityDto(**r)
        return r

    def Update(self, id: TKey, dto: TUpdateInput) -> TEntityDto:
        json = dto.model_dump() if dto is not None else None
        # print(req.model_dump())
        r = self._rest_put_method(endpoint=self.endpoint, uri=id, json=json)
        print(type(r))
        print(r)

        r = self._TEntityDto(**r)
        return r

    def Delete(self, id: TKey) -> bool:
        return self._rest_delete_method(endpoint=self.endpoint, uri=id)



"""apiclient.py"""

import sys
from rich.console import Console
from rich.progress import track

from .Api.AuthApi import Auth
from .Api.ExchangeApi import ExchangeApi

# minimal traceback
sys.tracebacklimit = 1


class ApiClient:
    """API Client"""

    Login: bool = False
    """
    Login indicate if logined.
    """

    Exchange: ExchangeApi = None

    def __init__(
        self,
        authTokenUrl: str = "https://www.sparkstrader.com/connect/token",
        apiUrl: str = "https://www.sparkstrader.com/api",
    ) -> None:
        self.authTokenUrl = authTokenUrl
        self.apiUrl = apiUrl
        self.console = Console()

    def Login(self, username: str, password: str) -> bool:
        """
        Login to the API using the specified username and password.

        Args:
            username: The username for authentication.
            password: The password for authentication.

        Returns:
            bool: True if the login is successful, False otherwise.

        Raises:
            None.
        """
        auth = Auth()
        authinfo = {
            "client_id": "Sparks_App",
            "scope": "Sparks",
            "grant_type": "password",
            "username": username,
            "password": password,
        }
        self.AccessToken = auth.GetAccessToken(self.authTokenUrl, authinfo)
        print(f"AccessToken:{self.AccessToken}")
        self.Login = self.AccessToken is not None
        if self.Login:
            self.Exchange = ExchangeApi(self.apiUrl, self.AccessToken)
        return self.Login



import sys
import requests
from json.decoder import JSONDecodeError
from requests import ConnectionError as requests_ConnectionError
from requests import Timeout as requests_Timeout
from requests.exceptions import HTTPError as requests_HTTPError
from rich.console import Console


class Auth:
    def __init__(self) -> None:
        self.console = Console()

    def GetAccessToken(self, endpoint: str, auth_info: dict) -> str:
        if not endpoint.strip():
            raise ValueError("endpoint is empty!")

        try:
            # 通过password flow模式，获取access token
            head = {"Content-Type": "application/x-www-form-urlencoded"}
            # auth_info = {
            #    "client_id": "Sparks_App",
            #    "scope": "Sparks",
            #    "grant_type": "password",
            #    "username": "admin",
            #    "password": "1q2w3E*",
            # }
            resp = requests.post(
                endpoint,
                data=auth_info,
                headers=head,
                verify=False,
            )

            if resp.status_code != 200:
                try:
                    if "message" in resp.json():
                        resp_message = resp.json()["message"]
                    elif "errors" in resp.json():
                        self.console.log(resp.json())
                        sys.exit(1)
                    else:
                        resp_message = ""

                    message = f"({resp.status_code}) {self._api_url} - {resp_message}"
                    self.console.log(message)

                except JSONDecodeError as err:
                    self.console.log(err)

            try:
                resp.raise_for_status()

                return resp.json()["access_token"]
            except ValueError as err:
                self.console.log(err)

        except requests_ConnectionError as err:
            self.console.log(err)
        except requests_HTTPError as err:
            self.console.log(err)
        except requests_Timeout as err:
            self.console.log(err)
        return None


# import sys

# sys.path.insert(0, f"{sys.path[0]}/../../")
# from sparks import config as cfg
#
# if __name__ == "__main__":
#    print(cfg.AuthTokenUrl)
#    auth = Auth()
#    accessToken = auth.GetAccessToken(cfg.AuthTokenUrl, cfg.Auth_Info)
#    print(accessToken)



import sys

# from ..sparks.ApiClient import ApiClient

sys.path.insert(0, f"{sys.path[0]}/..")
from sparkstrader.ApiClient import ApiClient

AuthTokenUrl = "https://localhost:44319/connect/token"
ApiUrl = "https://localhost:44319/api"

api = ApiClient(AuthTokenUrl, ApiUrl)
login = api.Login("admin", "1q2w3E*")
print(f"Login:{login}")



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/7/22 22:09
Desc: PYPI info file
"""
import re
import ast

import setuptools

from sparkstrader import __version__, __author__, __authoremail__

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

requirementsPath = "requirements.txt"

requirementslist = []
f = open(requirementsPath, "r")
for line in f:
    requirementslist.append(str.strip(line))
f.close()
# print(requirementslist)

# print(__version__)
# print(__author__)
# print(__authoremail__)


setuptools.setup(
    name="sparkstrader",
    version=__version__,
    author=__author__,
    author_email=__authoremail__,
    license="MIT",
    description="python api for sparkstrader.com",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/yacper/sparks_api_py",
    packages=setuptools.find_packages(),
    install_requires=requirementslist,
    package_data={"": ["*.py", "*.json", "*.pk", "*.js", "*.zip"]},
    keywords=[
        "stock",
        "option",
        "futures",
        "fund",
        "bond",
        "index",
        "air",
        "finance",
        "spider",
        "quant",
        "quantitative",
        "investment",
        "trading",
        "algotrading",
        "data",
    ],
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.8",
)



from BaseApi import BaseApi
from AuthApi import Auth
import pandas as pd
from uuid import UUID
import sys

import config as cfg

# sys.path.insert(0, f"{sys.path[0]}/..")
from Dto.ExchangeDto import ExchangeDto
from Dto.PagedAndSortedResultRequestDto import PagedAndSortedResultRequestDto
from Dto.CreateUpdateExchangeDto import *


class Exchanges(BaseApi):
    def __init__(self, accessToken: str) -> None:
        super(Exchanges, self).__init__(accessToken)
        self.endpoint = "app/exchange"

    def Get(self, id: UUID) -> ExchangeDto:
        r = self._rest_get_method(endpoint=self.endpoint, uri=id)
        # print(type(r))
        # print(r)
        return ExchangeDto(**r)

    def GetList(self, dto: PagedAndSortedResultRequestDto = None) -> pd.DataFrame:
        params = dto.model_dump() if dto is not None else None
        print(params)
        r = self._rest_get_method(endpoint=self.endpoint, params=params)
        l = r["items"]
        df = pd.DataFrame(l)
        # from_list(ExchangeDto.from_dict, r.get("ExchangeDto"))
        print(type(r))
        print(r)
        return df

    def Create(self, dto: CreateUpdateExchangeDto) -> ExchangeDto:
        json = dto.model_dump() if dto is not None else None
        # print(req.model_dump())
        r = self._rest_post_method(endpoint=self.endpoint, json=json)
        print(type(r))
        print(r)

        r = ExchangeDto(**r)
        return r

    def Update(self, id: UUID, dto: CreateUpdateExchangeDto) -> ExchangeDto:
        json = dto.model_dump() if dto is not None else None
        # print(req.model_dump())
        r = self._rest_put_method(endpoint=self.endpoint, uri=id, json=json)
        print(type(r))
        print(r)

        r = ExchangeDto(**r)
        return r

    def Delete(self, id: UUID) -> bool:
        return self._rest_delete_method(endpoint=self.endpoint, uri=id)


# sys.path.insert(0, f"{sys.path[0]}/../../")
import config as cfg


if __name__ == "__main__":
    auth = Auth()
    cfg.AccessToken = auth.GetAccessToken(cfg.AuthTokenUrl, cfg.Auth_Info)
    print(cfg.AccessToken)

    exchanges = Exchanges(cfg.AccessToken)
    # df = exchanges.Get("3a0e534b-6fc9-93c9-b8e1-f6919567e8ea")
    # df = exchanges.GetList()
    # df = exchanges.GetList(PagedAndSortedResultRequestDto(maxResultCount=2))

    createDto = CreateUpdateExchangeDto(
        code="test",
        name="test",
        district=EDistrict.CN,
        currency=ECurrency.CNY,
        iconPath="",
    )
    df = exchanges.Create(createDto)
    print(df)

    createDto = CreateUpdateExchangeDto(
        code="test2",
        name="test2",
        district=EDistrict.CN,
        currency=ECurrency.CNY,
        iconPath="",
    )
    df = exchanges.Update(df.id, createDto)
    # print(df.dtypes)
    print(df)

    r = exchanges.Delete(df.id)
    # print(df.dtypes)
    print(f"Delete {df.id} :{r}")



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/7/21 18:16
Desc: To test intention, just write test code here!
"""
import sys

sys.path.insert(0, f"{sys.path[0]}/..")
from tests.testApiClient import *


def test_GetList():
    df = api.Exchange.GetList()
    print(df)
    # assert df.shape[0] > 0


if __name__ == "__main__":
    test_GetList()



import pandas as pd
from uuid import UUID

from .CrudApi import CrudApi
from .AuthApi import Auth

# sys.path.insert(0, f"{sys.path[0]}/..")
from ..Dto.ExchangeDto import ExchangeDto
from ..Dto.PagedAndSortedResultRequestDto import PagedAndSortedResultRequestDto
from ..Dto.CreateUpdateExchangeDto import *


class ExchangeApi(
    CrudApi[
        UUID,
        ExchangeDto,
        PagedAndSortedResultRequestDto,
        CreateUpdateExchangeDto,
        CreateUpdateExchangeDto,
    ]
):
    """
    Exchange API for performing CRUD operations on exchanges.

        from sparks.ApiClient import ApiClient

        api = ApiClient()
        login = api.Login("admin", "1q2w3E*")
        print(f"Login:{login}")

        # get exchanges list
        df = api.Exchange.GetList()
        print(df)
    """

    def __init__(self, apiUrl: str, accessToken: str) -> None:
        CrudApi.__init__(self, apiUrl, accessToken, "app/exchange", ExchangeDto)


## from sparks import config as cfg
# from .. import config as cfg
#
# if __name__ == "__main__":
#    auth = Auth()
#    cfg.AccessToken = auth.GetAccessToken(cfg.AuthTokenUrl, cfg.Auth_Info)
#    print(cfg.AccessToken)
#
#    exchanges = ExchangeApi(cfg.AccessToken)
#    # df = exchanges.Get("3a0e534b-6fc9-93c9-b8e1-f6919567e8ea")
#    # df = exchanges.GetList()
#    # df = exchanges.GetList(PagedAndSortedResultRequestDto(maxResultCount=2))
#
#    createDto = CreateUpdateExchangeDto(
#        code="test",
#        name="test",
#        district=EDistrict.CN,
#        currency=ECurrency.CNY,
#        iconPath="",
#    )
#    df = exchanges.Create(createDto)
#    print(df)
#
#    createDto = CreateUpdateExchangeDto(
#        code="test2",
#        name="test2",
#        district=EDistrict.CN,
#        currency=ECurrency.CNY,
#        iconPath="",
#    )
#    df = exchanges.Update(df.id, createDto)
#    # print(df.dtypes)
#    print(df)
#
#    r = exchanges.Delete(df.id)
#    # print(df.dtypes)
#    print(f"Delete {df.id} :{r}")



from dataclasses import dataclass
from typing import Optional
from pydantic import BaseModel, Field
from sparkstrader.Enums.Enums import *
from datetime import datetime
from uuid import UUID


class ExchangeDto(BaseModel):
    code: str
    name: str
    district: EDistrict
    currency: ECurrency
    iconPath: Optional[str]
    isDeleted: bool
    deleterId: UUID | None
    deletionTime: datetime | None
    lastModificationTime: datetime | None
    lastModifierId: UUID | None
    creationTime: datetime | None
    creatorId: UUID | None
    id: UUID
